//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   EP123.v
//
// Abstract:
//   Endpoint 1,2,3 controller
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: fb1467f1551d3b43c1237500c83e0a1d70b678a2 $

`include "timescale.v"
`include "usbHostSlaveReg_define.v"

module EP123 (clk,
`ifdef USE_7SEG
disp7Seg,
`endif
initComplete, rst, wb_ack, wb_addr, wb_data_i, wb_data_o, wb_stb, wb_we, wbBusGnt, wbBusReq,stateConfigured);
input   clk;
`ifdef USE_7SEG
output [7:0] disp7Seg;
`endif
input   initComplete;
input   rst;
input   wb_ack;
input   [7:0]wb_data_i;
input   wbBusGnt;
output  [7:0]wb_addr;
output  [7:0]wb_data_o;
output  wb_stb;
output  wb_we;
output  wbBusReq;
input   stateConfigured;

wire    clk;
wire    initComplete;
wire    rst;
wire    wb_ack;
reg     [7:0]wb_addr, next_wb_addr;
wire    [7:0]wb_data_i;
reg     [7:0]wb_data_o, next_wb_data_o;
reg     wb_stb, next_wb_stb;
reg     wb_we, next_wb_we;
wire    wbBusGnt;
reg     wbBusReq, next_wbBusReq;

// diagram signals declarations
reg  [7:0]cnt, next_cnt;
reg dataSeq, next_dataSeq;
reg localRst, next_localRst;
reg transDone, next_transDone;
reg outMessage, next_outMessage;

localparam P_MESSAGE_KEY  = 8'h08;  // CTRL-H
localparam P_MESSAGE_SIZE = 6'd51;
   
function [7:0] f_message;
  input [5:0] cnt;
  reg [7:0]  ret;
  begin
    case (cnt)
      6'd51:ret = "O";
      6'd50:ret = "p";
      6'd49:ret = "e";
      6'd48:ret = "n";
      6'd47:ret = "C";
      6'd46:ret = "o";
      6'd45:ret = "r";
      6'd44:ret = "e";
      6'd43:ret = "s";
      6'd42:ret = " ";
      6'd41:ret = "U";
      6'd40:ret = "S";
      6'd39:ret = "B";
      6'd38:ret = " ";
      6'd37:ret = "I";
      6'd36:ret = "P";
      6'd35:ret = " ";
      6'd34:ret = "C";
      6'd33:ret = "o";
      6'd32:ret = "r";
      6'd31:ret = "e";
      6'd30:ret = " ";
      6'd29:ret = "C";
      6'd28:ret = "D";
      6'd27:ret = "C";
      6'd26:ret = " ";
      6'd25:ret = "A";
      6'd24:ret = "C";
      6'd23:ret = "M";
      6'd22:ret = " ";
      6'd21:ret = "S";
      6'd20:ret = "a";
      6'd19:ret = "m";
      6'd18:ret = "p";
      6'd17:ret = "l";
      6'd16:ret = "e";
      6'd15:ret = ".";
      6'd14:ret = "(";
      6'd13:ret = "L";
      6'd12:ret = "o";
      6'd11:ret = "g";
      6'd10:ret = "i";
      6'd9: ret = "c";
      6'd8: ret = " ";
      6'd7: ret = "o";
      6'd6: ret = "n";
      6'd5: ret = "l";
      6'd4: ret = "y";
      6'd3: ret = ")";
      6'd2: ret = "\r";
      6'd1: ret = "\n";
      default: ret = 8'd0;
    endcase
    f_message = ret;
  end
endfunction      
   
// BINARY ENCODED state machine: EP1St
// State codes definitions:
localparam P_START                        = 5'h00;
localparam P_INIT_TRANS_WT_GNT            = 5'h15;
localparam P_INIT_EP1_CONFIG              = 5'h16;
localparam P_INIT_EP2_CONFIG              = 5'h17;
localparam P_INIT_EP3_CONFIG              = 5'h18;
localparam P_INIT_TRANS_WT_UNGNT          = 5'h19;
localparam P_DO_TRANS_WT_GNT              =  5'h01;
localparam P_DO_TRANS_EP1_RX_EMPTY        = 5'h02;
localparam P_DO_TRANS_EP2_TX_EMPTY        = 5'h03;
localparam P_DO_TRANS_TRANS_GO            = 5'h04;
localparam P_DO_TRANS_WT_TRANS_DONE_WT_GNT = 5'h05;
localparam P_DO_TRANS_WT_TRANS_DONE_GET_RDY_STS=  5'h06;
localparam P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT = 5'h07;
localparam P_DO_TRANS_WT_TRANS_DONE_CHK_DONE = 5'h08;
localparam P_DO_TRANS_CDC_WT_GNT             = 5'h09;
localparam P_DO_TRANS_CDC_EP1_RX_SIZE1       = 5'h0a;
localparam P_DO_TRANS_CDC_EP1_RX_SIZE2       = 5'h0b;
localparam P_DO_TRANS_CDC_EP1_RX_FIFO        = 5'h0c;
localparam P_DO_TRANS_CDC_EP2_TX_FIFO        = 5'h0d;
localparam P_DO_TRANS_EP2_ENABLE             = 5'h0e;
localparam P_DO_TRANS_EP2_TRANS_DONE_WT_GNT  = 5'h0f;
localparam P_DO_TRANS_EP2_TRANS_DONE_GET_RDY_STS = 5'h10;
localparam P_DO_TRANS_EP2_TRANS_DONE_WT_UNGNT = 5'h11;
localparam P_DO_TRANS_EP2_TRANS_DONE_CHK_DONE = 5'h12;
   
localparam P_DO_TRANS_WT_TRANS_DONE_DEL       = 5'h13;
localparam P_DO_TRANS_EP2_TRANS_DONE_DEL      = 5'h14;
localparam P_DO_TRANS_EP1_STATUS              = 5'h1a;
localparam P_DO_TRANS_EP2_STATUS              = 5'h1b;
localparam P_DO_TRANS_EP3_STATUS              = 5'h1c;

reg [4:0] CurrState_EPxST, NextState_EPxST;
reg  [15:0]rxDataSize, next_rxDataSize;
// Diagram actions (continuous assignments allowed only: assign ...)
// diagram ACTION


// Machine: EPxST

// NextState logic (combinatorial)
always @*
begin
  NextState_EPxST = CurrState_EPxST;
  // Set default values for outputs and signals
  next_wbBusReq = wbBusReq;
  next_wb_addr = wb_addr;
  next_wb_data_o = wb_data_o;
  next_wb_stb = wb_stb;
  next_wb_we = wb_we;
  next_dataSeq = dataSeq;
  next_transDone = transDone;
  next_cnt = cnt;
  next_rxDataSize = rxDataSize;
  next_outMessage = outMessage;
  case (CurrState_EPxST)  // synopsys parallel_case full_case
    P_START:
    begin
      next_wbBusReq = 1'b0;
      next_wb_addr = 8'h00;
      next_wb_data_o = 8'h00;
      next_wb_stb = 1'b0;
      next_wb_we = 1'b0;
      next_cnt = 8'h00;
      next_dataSeq = 1'b0;
      next_transDone = 1'b0;
      if (initComplete == 1'b1)
      begin
        NextState_EPxST = P_INIT_TRANS_WT_GNT;
      end
    end
    P_INIT_TRANS_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EPxST = P_INIT_EP1_CONFIG;
      end
    end
    P_INIT_EP1_CONFIG:
    begin
      next_wb_addr = `RA_EP1_CONTROL_REG;
      next_wb_data_o = 8'h01;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        next_wb_stb = 1'b0;
        NextState_EPxST = P_INIT_EP2_CONFIG;
      end
    end
    P_INIT_EP2_CONFIG:
    begin
      next_wb_addr = `RA_EP2_CONTROL_REG;
      next_wb_data_o = 8'h01;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        next_wbBusReq = 1'b0;
        NextState_EPxST = P_INIT_EP3_CONFIG;
      end
    end
    P_INIT_EP3_CONFIG:
    begin
      next_wb_addr = `RA_EP3_CONTROL_REG;
      next_wb_data_o = 8'h01;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        next_wbBusReq = 1'b0;
        NextState_EPxST =  P_INIT_TRANS_WT_UNGNT;
      end
    end
    P_INIT_TRANS_WT_UNGNT:
    begin
      if (wbBusGnt == 1'b0)
      begin
        NextState_EPxST = P_DO_TRANS_WT_GNT;
      end
    end
    P_DO_TRANS_WT_GNT:
    begin
      if (stateConfigured) begin  
        next_wbBusReq = 1'b1;
        if (wbBusGnt == 1'b1)
        begin
          NextState_EPxST = P_DO_TRANS_EP1_RX_EMPTY;
        end
      end
    end
    P_DO_TRANS_EP1_RX_EMPTY:
    begin
      next_wb_addr = `RA_EP1_RX_FIFO_CONTROL_REG;
      next_wb_data_o = 8'h01;
      //force rx fifo empty
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TX_EMPTY;
        next_wb_stb = 1'b0;
        next_wb_addr = `RA_EP2_TX_FIFO_CONTROL_REG;
        next_wb_we = 1'b1;
      end
    end
    P_DO_TRANS_EP2_TX_EMPTY:
    begin
      next_wb_addr = `RA_EP2_TX_FIFO_CONTROL_REG;
      next_wb_data_o = 8'h01;
      //force tx fifo empty
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_TRANS_GO;
        next_wb_stb = 1'b0;
      end
    end
    P_DO_TRANS_TRANS_GO:
    begin
      next_wb_addr = `RA_EP1_CONTROL_REG;
      next_wb_data_o = 8'h03;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_WT_GNT;
        next_wb_stb = 1'b0;
        next_transDone = 1'b0;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_GET_RDY_STS;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_GET_RDY_STS:
    begin
      next_wb_addr = `RA_EP1_CONTROL_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
//        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT;
        NextState_EPxST = P_DO_TRANS_EP1_STATUS;
        next_wb_stb = 1'b0;
        next_transDone = ~wb_data_i[`ENDPOINT_READY_BIT];
      end
    end
    P_DO_TRANS_EP1_STATUS:
    begin
      next_wb_addr = `RA_EP1_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT;
        next_wb_stb = 1'b0;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT:
    begin
      next_wbBusReq = 1'b0;
      if (wbBusGnt == 1'b0)
      begin
        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_CHK_DONE;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_CHK_DONE:
    begin
      if (transDone == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_CDC_WT_GNT;
      end
      else
      begin
        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_DEL;
        next_cnt = 8'h00;
      end
    end
    P_DO_TRANS_CDC_WT_GNT :
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_CDC_EP1_RX_SIZE1;
      end
    end
    P_DO_TRANS_CDC_EP1_RX_SIZE1 :
      begin
      next_wb_addr = `RA_EP1_RX_FIFO_DATA_COUNT_MSB;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_CDC_EP1_RX_SIZE2;
        next_wb_stb = 1'b0;
        next_rxDataSize[15:8] = wb_data_i;
      end
    end
    P_DO_TRANS_CDC_EP1_RX_SIZE2 :
      begin
      next_wb_addr = `RA_EP1_RX_FIFO_DATA_COUNT_LSB;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_CDC_EP1_RX_FIFO;
        next_wb_stb = 1'b0;
        next_rxDataSize[7:0] = wb_data_i;
      end
    end
    P_DO_TRANS_CDC_EP1_RX_FIFO :
      begin
      next_wb_addr = `RA_EP1_RX_FIFO_DATA_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_CDC_EP2_TX_FIFO;
        next_wb_stb = 1'b0;
        if ((wb_data_i == P_MESSAGE_KEY)&& !outMessage) begin
          next_outMessage = 1'b1;
          next_wb_data_o = f_message(P_MESSAGE_SIZE);
          next_rxDataSize = P_MESSAGE_SIZE - 1'b1;
	end else begin
          next_wb_data_o = wb_data_i;
          next_rxDataSize = rxDataSize - 1'b1;
        end
      end
    end
    P_DO_TRANS_CDC_EP2_TX_FIFO :
      begin
      next_wb_addr = `RA_EP2_TX_FIFO_DATA_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        if (rxDataSize == 16'h0) begin
          NextState_EPxST = P_DO_TRANS_EP2_ENABLE;
          next_wb_stb = 1'b0;
          next_outMessage = 1'b0;
	end else begin
          if (outMessage) begin
            next_wb_stb = 1'b1;
            next_rxDataSize = rxDataSize - 1'b1;
            next_wb_data_o = f_message(rxDataSize[5:0]);
	  end else begin
            NextState_EPxST = P_DO_TRANS_CDC_EP1_RX_FIFO;
            next_wb_stb = 1'b0;
          end
	end
      end
    end
    P_DO_TRANS_EP2_ENABLE:
    begin
      next_wb_addr = `RA_EP2_CONTROL_REG;
      if (dataSeq == 1'b1)
        next_wb_data_o = 8'h07;
      else
        next_wb_data_o = 8'h03;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_WT_GNT;
        next_wb_stb = 1'b0;
        next_transDone = 1'b0;
      end
    end
    P_DO_TRANS_EP2_TRANS_DONE_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_GET_RDY_STS;
      end
    end
    P_DO_TRANS_EP2_TRANS_DONE_GET_RDY_STS:
    begin
      next_wb_addr = `RA_EP2_CONTROL_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
//        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_WT_UNGNT;
        NextState_EPxST = P_DO_TRANS_EP2_STATUS;
        next_wb_stb = 1'b0;
        next_transDone = ~wb_data_i[`ENDPOINT_READY_BIT];
      end
    end
    P_DO_TRANS_EP2_STATUS:
    begin
      next_wb_addr = `RA_EP2_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_WT_UNGNT;
        next_wb_stb = 1'b0;
      end
    end
    P_DO_TRANS_EP2_TRANS_DONE_WT_UNGNT:
    begin
      next_wbBusReq = 1'b0;
      if (wbBusGnt == 1'b0)
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_CHK_DONE;
      end
    end
    P_DO_TRANS_EP2_TRANS_DONE_CHK_DONE:
    begin
      if (transDone == 1'b1)
      begin
        if (dataSeq == 1'b1)
          next_dataSeq = 1'b0;
        else
          next_dataSeq = 1'b1;
        NextState_EPxST = P_DO_TRANS_WT_GNT;
      end
      else
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_DEL;
        next_cnt = 8'h00;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_DEL:
    begin
      next_cnt = cnt + 1'b1;
      if (cnt == `ONE_USEC_DEL)
      begin
        NextState_EPxST = P_DO_TRANS_WT_TRANS_DONE_WT_GNT;
      end
    end
    P_DO_TRANS_EP2_TRANS_DONE_DEL:
    begin
      next_cnt = cnt + 1'b1;
      if (cnt == `ONE_USEC_DEL)
      begin
        NextState_EPxST = P_DO_TRANS_EP2_TRANS_DONE_WT_GNT;
      end
    end
  endcase
end


   
// Current State Logic (sequential)
always @ (posedge clk)
begin
  if (rst == 1'b1)
    CurrState_EPxST <= P_START;
  else
    CurrState_EPxST <= NextState_EPxST;
end

always @ (posedge clk)
begin
  rxDataSize <= next_rxDataSize;
end
   
// Registered outputs logic
always @ (posedge clk)
begin
  if (rst == 1'b1)
  begin
    wbBusReq <= 1'b0;
    wb_addr <= 8'h00;
    wb_data_o <= 8'h00;
    wb_stb <= 1'b0;
    wb_we <= 1'b0;
    dataSeq <= 1'b0;
    transDone <= 1'b0;
    cnt <= 8'h00;
    outMessage <= 1'b0;
  end
  else 
  begin
    wbBusReq <= next_wbBusReq;
    wb_addr <= next_wb_addr;
    wb_data_o <= next_wb_data_o;
    wb_stb <= next_wb_stb;
    wb_we <= next_wb_we;
    dataSeq <= next_dataSeq;
    transDone <= next_transDone;
    cnt <= next_cnt;
    outMessage <= next_outMessage;
  end
end

`ifdef USE_7SEG
reg [7:0] disp7Seg;
wire w_update7SEG; 
assign w_update7SEG = (CurrState_EPxST == P_DO_TRANS_CDC_EP1_RX_FIFO) & wb_ack;
always @ (posedge clk) begin
  if (rst == 1'b1) disp7Seg <= 8'h00;
  else if (w_update7SEG) disp7Seg <= f_7segEncode(next_wb_data_o);
end

function [7:0] f_7segEncode;
  input [7:0] c;
  reg [7:0] ret;
  begin
   //            bit pos
   //     -        0
   //    | |      5 1
   //     -        6
   //    | |      4 2
   //     -        3
    case(c)
      "0": ret = ~8'h3f;
      "1": ret = ~8'h06;
      "2": ret = ~8'h5b;
      "3": ret = ~8'h4f;
      "4": ret = ~8'h66;
      "5": ret = ~8'h6d;
      "6": ret = ~8'h7d;
      "7": ret = ~8'h27;
      "8": ret = ~8'h7f;
      "9": ret = ~8'h67;
      "A",
      "a": ret = ~8'h77;
      "B",
      "b": ret = ~8'h7c;
      "C",
      "c": ret = ~8'h58;
      "D",
      "d": ret = ~8'h5e;
      "E",
      "e": ret = ~8'h79;
      "F",
      "f": ret = ~8'h71;
      "G",
      "g": ret = ~8'h3d;
      "H",
      "h": ret = ~8'h74;
      "I",
      "i": ret = ~8'h04;
      "J",
      "j": ret = ~8'h1e;
      "K",
      "k": ret = ~8'h75;
      "L",
      "l": ret = ~8'h38;
      "M",
      "m": ret = ~8'h37;
      "N",
      "n": ret = ~8'h54;
      "O",
      "o": ret = ~8'h5c;
      "P",
      "p": ret = ~8'h73;
      "Q",
      "q": ret = ~8'h7b;
      "R",
      "r": ret = ~8'h50;
      "S",
      "s": ret = ~8'h6c;
      "T",
      "t": ret = ~8'h78;
      "U",
      "u": ret = ~8'h1c;
      "V",
      "v": ret = ~8'h3e;
      "W",
      "w": ret = ~8'h7e;
      "X",
      "x": ret = ~8'h76;
      "Y",
      "y": ret = ~8'h6e;
      "Z",
      "Z": ret = ~8'h1b;
  default:
           ret = ~8'h00;
    endcase
    f_7segEncode = ret;
  end
endfunction
`endif

endmodule
