//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   ckLineStateCdc.v
//
// Abstract:
//   USB connect/disconnect detection
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 33e7ff17a9ea44de421d8fd792bac045dbe857ea $

`include "timescale.v"
`include "usbSlaveControl_h.v"
`include "usbHostSlaveReg_define.v"
`include "usbSerialInterfaceEngine_h.v"
`include "usbCdcDevice_define.v"
module ckLineStateCdc (clk, initComplete, rst, usbRstDet, wb_ack, wb_addr, wb_data_i, wb_stb, wb_we, wbBusGnt, wbBusReq);
input   clk;
input   initComplete;
input   rst;
input   wb_ack;
input   [7:0]wb_data_i;
input   wbBusGnt;
output  usbRstDet;
output  [7:0]wb_addr;
output  wb_stb;
output  wb_we;
output  wbBusReq;

wire    clk;
wire    initComplete;
wire    rst;
reg     usbRstDet, next_usbRstDet;
wire    wb_ack;
reg     [7:0]wb_addr, next_wb_addr;
wire    [7:0]wb_data_i;
reg     wb_stb, next_wb_stb;
reg     wb_we, next_wb_we;
wire    wbBusGnt;
reg     wbBusReq, next_wbBusReq;

// diagram signals declarations
reg  [15:0]cnt, next_cnt;
reg  [1:0]resetState, next_resetState;

// BINARY ENCODED state machine: chkLSt
// State codes definitions:
localparam P_START = 3'b000;
localparam P_GET_STAT = 3'b001;
localparam P_WT_GNT = 3'b010;
localparam P_SET_RST_DET = 3'b011;
localparam P_DEL_ONE_MSEC = 3'b100;

reg [2:0]CurrState_chkLSt, NextState_chkLSt;

// Diagram actions (continuous assignments allowed only: assign ...)
// diagram ACTION


// Machine: chkLSt

// NextState logic (combinatorial)
always @*
begin
  NextState_chkLSt = CurrState_chkLSt;
  // Set default values for outputs and signals
  next_usbRstDet = usbRstDet;
  next_wbBusReq = wbBusReq;
  next_wb_addr = wb_addr;
  next_wb_stb = wb_stb;
  next_wb_we = wb_we;
  next_cnt = cnt;
  next_resetState = resetState;
  case (CurrState_chkLSt)  // synopsys parallel_case full_case
    P_START:
    begin
      next_usbRstDet = 1'b0;
      next_wbBusReq = 1'b0;
      next_wb_addr = 8'h00;
      next_wb_stb = 1'b0;
      next_wb_we = 1'b0;
      next_cnt = 16'h0000;
      next_resetState = 2'b00;
      if (initComplete == 1'b1)
      begin
        NextState_chkLSt = P_WT_GNT;
      end
    end
    P_GET_STAT:
    begin
      next_wb_addr = `RA_SC_LINE_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_chkLSt = P_SET_RST_DET;
        next_wb_stb = 1'b0;
        // `DISCONNECT = 0 : same as RESET, defined in usbSerialInterfaceEngine.v
        if ( (wb_data_i[1:0] == `DISCONNECT) || (wb_data_i[`VBUS_PRES_BIT] == 1'b0) )
        next_resetState = {resetState[0], 1'b1};
        else
        next_resetState = 2'b00;
        next_wbBusReq = 1'b0;
      end
    end
    P_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_chkLSt = P_GET_STAT;
      end
    end
    P_SET_RST_DET:
    begin
      NextState_chkLSt = P_DEL_ONE_MSEC;
      if (resetState == 2'b11) // if reset condition aserted for 2mS
        next_usbRstDet = 1'b1;
      next_cnt = 16'h0000;
    end
    P_DEL_ONE_MSEC:
    begin
      next_cnt = cnt + 1'b1;
      next_usbRstDet = 1'b0;
      if (cnt == `ONE_MSEC_DEL)
      begin
        NextState_chkLSt = P_WT_GNT;
      end
    end
  endcase
end

// Current State Logic (sequential)
always @ (posedge clk)
begin
  if (rst == 1'b1)
    CurrState_chkLSt <= P_START;
  else
    CurrState_chkLSt <= NextState_chkLSt;
end

// Registered outputs logic
always @ (posedge clk)
begin
  if (rst == 1'b1)
  begin
    usbRstDet <= 1'b0;
    wbBusReq <= 1'b0;
    wb_addr <= 8'h00;
    wb_stb <= 1'b0;
    wb_we <= 1'b0;
    cnt <= 16'h0000;
    resetState <= 2'b00;
  end
  else 
  begin
    usbRstDet <= next_usbRstDet;
    wbBusReq <= next_wbBusReq;
    wb_addr <= next_wb_addr;
    wb_stb <= next_wb_stb;
    wb_we <= next_wb_we;
    cnt <= next_cnt;
    resetState <= next_resetState;
  end
end

endmodule
