//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   usbCdcTarget.v
//
// Abstract:
//   USB CDC logic+processor version IP top
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 3b6c4212fbf0e0c030cea0c6a6ee5ed8b54d78d0 $

module usbCdcTarget (
  // Wishbone
  clk_i,
  rst_i,
  address_i, 
  data_i, 
  data_o, 
  we_i, 
  strobe_i,
  ack_o,
  int_o,
  // usb state
  stateDefault,
  stateAddress,
  stateConfigured,
  // USB
  usbClk,
  USBWireDataIn,
  USBWireDataOut,
  USBWireCtrlOut,
  USBFullSpeed,
  usbDPlusPullup,
  usbDMinusPullup,
  vBusDetect
);
// Wishbone
input clk_i;
input rst_i;
input [7:0] address_i;
input [7:0] data_i;
output [7:0] data_o;
input we_i;
input strobe_i;
output ack_o;
output int_o;
  // usb state
output stateDefault;
output stateAddress;
output stateConfigured;
// USB
input usbClk;
input [1:0] USBWireDataIn;
output [1:0] USBWireDataOut;
output USBWireCtrlOut;
output USBFullSpeed;
output usbDPlusPullup;
output usbDMinusPullup;
input vBusDetect;

//local wires and regs
wire [7:0] wb_addr0;
wire wb_stb0;
wire wb_we0;
wire wbBusReq0;
wire wbBusGnt0;
wire [7:0] wb_addr1;
wire [7:0] wb_data_o1;
wire wb_stb1;
wire wb_we1;
wire wbBusReq1;
wire wbBusGnt1;
wire [7:0] wb_addr2;
wire [7:0] wb_data_o2;
wire wb_stb2;
wire wb_we2;
wire wbBusReq2;
wire wbBusGnt2;
wire [7:0] wb_adr;
wire [7:0] wb_dat_to_usb;
wire [7:0] wb_dat_from_usb;
wire wb_we;
wire wb_stb;
wire wb_ack;
reg [1:0] resetReg;
wire initComplete;
wire usbRstDet;
wire [7:0] memAddr;
wire [7:0] memData;
wire USBWireCtrlOut;
wire [1:0] USBWireDataIn;
wire [1:0] USBWireDataOut;

wire w_slaveSOFRxedIntOut;
wire w_slaveResetEventIntOut;
wire w_slaveResumeIntOut;
wire w_slaveTransDoneIntOut;
wire w_slaveNAKSentIntOut;
wire w_slaveVBusDetIntOut;

//Parameters declaration: 
defparam usbSlaveInst.EP0_FIFO_DEPTH = 64;
defparam usbSlaveInst.EP0_FIFO_ADDR_WIDTH = 6;
defparam usbSlaveInst.EP1_FIFO_DEPTH = 64;
defparam usbSlaveInst.EP1_FIFO_ADDR_WIDTH = 6;
defparam usbSlaveInst.EP2_FIFO_DEPTH = 64;
defparam usbSlaveInst.EP2_FIFO_ADDR_WIDTH = 6;
defparam usbSlaveInst.EP3_FIFO_DEPTH = 64;
defparam usbSlaveInst.EP3_FIFO_ADDR_WIDTH = 6;
usbSlave usbSlaveInst (
  .clk_i(clk_i),
  .rst_i(rst_i),
  .address_i(wb_adr),
  .data_i(wb_dat_to_usb),
  .data_o(wb_dat_from_usb),
  .we_i(wb_we),
  .strobe_i(wb_stb),
  .ack_o(wb_ack),
  .usbClk(usbClk),
  .slaveSOFRxedIntOut(w_slaveSOFRxedIntOut),
  .slaveResetEventIntOut(w_slaveResetEventIntOut),
  .slaveResumeIntOut(w_slaveResumeIntOut),
  .slaveTransDoneIntOut(w_slaveTransDoneIntOut),
  .slaveNAKSentIntOut(w_slaveNAKSentIntOut),
  .slaveVBusDetIntOut(w_slaveVBusDetIntOut),
  .USBWireDataIn(USBWireDataIn),
  .USBWireDataInTick(),
  .USBWireDataOut(USBWireDataOut),
  .USBWireDataOutTick(),
  .USBWireCtrlOut(USBWireCtrlOut),
  .USBFullSpeed(USBFullSpeed),
  .USBDPlusPullup(usbDPlusPullup),
  .USBDMinusPullup(usbDMinusPullup),
  .vBusDetect(vBusDetect)
);

ckLineStateCdc u_ckLineStateCdc (
  .clk(clk_i),
  .rst(rst_i),
  .initComplete(initComplete),
  .usbRstDet(usbRstDet),
  .wb_ack(wb_ack),
  .wb_addr(wb_addr0),
  .wb_data_i(wb_dat_from_usb),
  .wb_stb(wb_stb0),
  .wb_we(wb_we0),
  .wbBusGnt(wbBusGnt0),
  .wbBusReq(wbBusReq0)
);


EP0Cdc u_EP0Cdc (
  .clk(clk_i), 
  .rst(rst_i | usbRstDet),
  .initComplete(initComplete),
  .wb_ack(wb_ack),
  .wb_addr(wb_addr1),
  .wb_data_i(wb_dat_from_usb),
  .wb_data_o(wb_data_o1),
  .wb_stb(wb_stb1),
  .wb_we(wb_we1),
  .wbBusGnt(wbBusGnt1),
  .wbBusReq(wbBusReq1),
  .memAddr(memAddr),
  .memData(memData),
  .memRdEn(),
  .stateDefault(stateDefault),
  .stateAddress(stateAddress),
  .stateConfigured(stateConfigured)
);

usbCdcROM u_usbROM (
  .clk(clk_i),
  .addr(memAddr),
  .data(memData)
);


wbIF u_wbIF (
  .clk(clk_i),
  .rst(rst_i),
// Outside
  .address_i(address_i), 
  .data_i(data_i), 
  .data_o(data_o), 
  .we_i(we_i), 
  .strobe_i(strobe_i),
  .ack_o(ack_o),
  .int_o(int_o),
// Internal
  .initComplete(initComplete),
  .stateConfigured(stateConfigured),
  .wb_ack(wb_ack),
  .wb_addr(wb_addr2),
  .wb_data_i(wb_dat_from_usb),
  .wb_data_o(wb_data_o2),
  .wb_stb(wb_stb2),
  .wb_we(wb_we2),
  .wbBusGnt(wbBusGnt2),
  .wbBusReq(wbBusReq2),
//  interrupt
  .slaveSOFRxedIntOut(w_slaveSOFRxedIntOut),
  .slaveResetEventIntOut(w_slaveResetEventIntOut),
  .slaveResumeIntOut(w_slaveResumeIntOut),
  .slaveTransDoneIntOut(w_slaveTransDoneIntOut),
  .slaveNAKSentIntOut(w_slaveNAKSentIntOut),
  .slaveVBusDetIntOut(w_slaveVBusDetIntOut)
);

wbArbCdc u_wbArbCdc (
  .clk(clk_i),
  .rst(rst_i),

  .addr0_i(wb_addr0),
  .data0_i(8'h00),
  .stb0_i(wb_stb0),
  .we0_i(wb_we0),
  .req0(wbBusReq0),
  .gnt0(wbBusGnt0),

  .addr1_i(wb_addr1),
  .data1_i(wb_data_o1),
  .stb1_i(wb_stb1),
  .we1_i(wb_we1),
  .req1(wbBusReq1),
  .gnt1(wbBusGnt1),

  .addr2_i(wb_addr2),
  .data2_i(wb_data_o2),
  .stb2_i(wb_stb2),
  .we2_i(wb_we2),
  .req2(wbBusReq2),
  .gnt2(wbBusGnt2),

  .addr_o(wb_adr),
  .data_o(wb_dat_to_usb),
  .stb_o(wb_stb),
  .we_o(wb_we)
);


endmodule

