//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   wbIF.v
//
// Abstract:
//   Wishbone - Processor bus bridge
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: f8f3940d8b54fdec8a887cabc32cefd23e8b0598 $

`include "timescale.v"
`include "usbHostSlaveReg_define.v"

module wbIF (
clk,
rst,
// Outside
address_i, 
data_i, 
data_o, 
we_i, 
strobe_i,
ack_o,
int_o,
// Internal
initComplete,
stateConfigured,
wb_ack,
wb_addr,
wb_data_i,
wb_data_o,
wb_stb,
wb_we,
wbBusGnt,
wbBusReq,
//  interrupt
slaveSOFRxedIntOut,
slaveResetEventIntOut,
slaveResumeIntOut,
slaveTransDoneIntOut,
slaveNAKSentIntOut,
slaveVBusDetIntOut
);

input   clk;
input   rst;
// Outside
input   [7:0] address_i;
input   [7:0] data_i;
output  [7:0] data_o;
input         we_i;
input         strobe_i;
output        ack_o;
output        int_o;

// Internal
input   initComplete;
input stateConfigured;
input   wb_ack;
input   [7:0]wb_data_i;
input   wbBusGnt;
output  [7:0]wb_addr;
output  [7:0]wb_data_o;
output  wb_stb;
output  wb_we;
output  wbBusReq;
//  interrupt
input   slaveSOFRxedIntOut;
input   slaveResetEventIntOut;
input   slaveResumeIntOut;
input   slaveTransDoneIntOut;
input   slaveNAKSentIntOut;
input   slaveVBusDetIntOut;

// state definition
localparam P_IDLE = 3'd0;
localparam P_WT_REQ = 3'd1;
localparam P_INTERNAL_REG = 3'd2;
localparam P_WT_GNT = 3'd3;
localparam P_WB_STROBE = 3'd4;
localparam P_UNGNT = 3'd5;

// New register address
localparam P_CONF_INT_MASK = 8'hf0;
localparam P_CONF_STATUS = 8'hf1;

// register
reg  [7:0] wb_addr;
reg  [7:0] wb_data_o;
reg        wb_stb;
reg        wb_we;
reg        wbBusReq;
reg  [7:0] data_o;

reg r_int;
reg [2:0] WBST;
reg ack_o;

reg r_int_mask;
reg r_configured;
reg stateConfigured_1z;
// wire
wire w_int;
wire [7:0] regData;
wire isInternal;
wire w_stateConfigured;
// assign
assign isInternal = (address_i == P_CONF_INT_MASK) |
                    (address_i == P_CONF_STATUS);
   
assign  regData   = (address_i == P_CONF_INT_MASK) ? {7'b0,r_int_mask} :
                                                     {7'b0,stateConfigured};

assign int_o = r_int & (!r_int_mask);
assign w_int = slaveSOFRxedIntOut |
               slaveResetEventIntOut |
               slaveResumeIntOut |
               slaveTransDoneIntOut |
               slaveNAKSentIntOut |
               slaveVBusDetIntOut;
 
assign w_stateConfigured = stateConfigured & ~stateConfigured_1z;

// always
always @(posedge clk) begin
  if (rst) begin
    WBST <= P_WT_REQ;
    wbBusReq <= 1'b0;
    wb_stb <= 1'b0;
    ack_o <= 1'b0;
  end else begin
    case (WBST)
      P_WT_REQ: begin
        ack_o <= 1'b0;
        if (strobe_i) begin
          if (isInternal) begin
            WBST <= P_INTERNAL_REG;
            ack_o <= 1'b1;
          end else begin
            wbBusReq <= 1'b1;
            WBST <= P_WT_GNT;
          end
        end
      end
      P_INTERNAL_REG: begin
        WBST <= P_WT_REQ;
        ack_o <= 1'b0;
      end
      P_WT_GNT: begin
        if (wbBusGnt) begin
          wb_stb <= 1'b1;
          WBST <= P_WB_STROBE;
        end
      end
      P_WB_STROBE: begin
        if (wb_ack) begin
          wb_stb <= 1'b0;
          ack_o <= 1'b1;
          WBST <= P_UNGNT;
        end
      end
      P_UNGNT: begin
        wbBusReq <= 1'b0;
        ack_o <= 1'b0;
        WBST <= P_WT_REQ;
      end
    endcase
  end
end

always @(posedge clk) begin
  case (WBST)
      P_WT_REQ: begin
        if (strobe_i) begin
          if (isInternal) begin
            if (~we_i) data_o <= regData;
          end
        end
      end
    P_WT_GNT: begin
      if (wbBusGnt) begin
        wb_addr <= address_i;
        wb_data_o <= data_i;
        wb_we <= we_i;
      end
    end
    P_WB_STROBE: begin
      if (wb_ack) begin
        if (!wb_we) data_o <= wb_data_i;
      end
    end
  endcase
end

always @(posedge clk or posedge  rst) begin
  if (rst) begin
    r_int_mask <= 1'b1;
  end else begin
    case (WBST)
      P_INTERNAL_REG: begin
        if (we_i) begin
          if (address_i == P_CONF_INT_MASK) r_int_mask <= data_i[0];
        end
      end
    endcase
  end
end

always @(posedge clk or posedge  rst) begin
  if (rst) begin
    r_int <= 1'b0;
  end else begin
    if (w_stateConfigured) begin
      r_int <= 1'b1;
    end else begin
      case (WBST)
        P_INTERNAL_REG: begin
          if (~we_i) begin
            if (address_i == P_CONF_STATUS) r_int <= 1'b0;
          end
          //if (~we_i) begin
          //  if (address_i == P_CONF_STATUS) r_int <= 1'b0;
          //end else begin
          //  if ((address_i == P_CONF_STATUS) & (data_i == 8'h5a)) r_int <= 1'b1;
	  //end
        end
      endcase
    end
  end
end
   
always @(posedge clk or posedge  rst) begin
  if (rst) begin
    stateConfigured_1z <= 1'b0;
  end else begin
    stateConfigured_1z <= stateConfigured;
  end
end

endmodule
